using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace PhoenixDLL.GUI.Seekers.Root
{
    /// <summary>
    /// Fentre de base pour une recherche
    /// </summary>
    public partial class fr_RootSeeker : Form
    {
        #region Variables
        /// <summary>
        /// Thread de recherche
        /// </summary>
        protected Thread th_Seeker;
        /// <summary>
        /// Search options
        /// </summary>
        private SearchOption vSearchOption = SearchOption.BeginsWith;
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        public fr_RootSeeker()
        {
            InitializeComponent();

            base.DialogResult = DialogResult.Cancel;
        }
        #endregion

        #region Event
        /// <summary>
        /// Seek options
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_SeekOptions(object sender, EventArgs e)
        {
            if (sender == tsmi_BeginsWith)
            {
                tsmi_BeginsWith.Checked = true;
                tsmi_Contains.Checked = false;
                tsdd_SeekType.Text = "Recherche: \"Commence par\"";
                this.SearchOption = SearchOption.BeginsWith;
            }
            else
            {
                tsmi_BeginsWith.Checked = false;
                tsmi_Contains.Checked = true;
                tsdd_SeekType.Text = "Recherche: \"Contient\"";
                this.SearchOption = SearchOption.Contains;
            }
            OnSearchOptionsChanged();

        }
        /// <summary>
        /// Evnement sur un double clique dans le DataGridView
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Dgv_DoubleClick(object sender, EventArgs e)
        {
            ExecuteDoubleClick();
        }
        /// <summary>
        /// Clique droit sur le DataGridViex
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DGV_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                DataGridView.HitTestInfo HitTestInfo = this.dgv_Results.HitTest(e.X, e.Y);
                if (HitTestInfo.RowIndex > -1)
                {
                    this.dgv_Results.SelectedRows[0].Selected = false;
                    this.dgv_Results.Rows[HitTestInfo.RowIndex].Selected = true;
                }
            }
        }
        /// <summary>
        /// Une touche du clavier  t presse
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_KeyPressed(object sender, KeyEventArgs e)
        {
            ExecuteKeyPressed(e);
        }
        /// <summary>
        /// Demande d'ajout
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_Add(object sender, EventArgs e) { ExecuteAdd(); }
        /// <summary>
        /// Demande de suppression
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_Del(object sender, EventArgs e) { ExecuteDel(); }
        #endregion

        #region Personal Event
        /// <summary>
        /// The search options changed
        /// </summary>
        public event EventHandler SearchOptionsChanged;
        /// <summary>
        /// Notify a search options changed
        /// </summary>
        protected virtual void OnSearchOptionsChanged()
        {
            if (SearchOptionsChanged != null)
                SearchOptionsChanged(this, new EventArgs());
        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Traitement du double clique sur le DataGridView
        /// </summary>
        protected virtual void ExecuteDoubleClick()
        {
            if (dgv_Results.SelectedRows.Count > 0)
                base.DialogResult = DialogResult.OK;
        }
        /// <summary>
        /// Code d'excution d'un appui sur une touche du clavier
        /// </summary>
        protected void ExecuteKeyPressed(KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                if (dgv_Results.SelectedRows.Count < 1) return;
                if (this.SearchOption == SearchOption.Contains) return;
                if (dgv_Results.Rows.Count == 1)
                    ExecuteDoubleClick();
            }
        }
        /// <summary>
        /// Excution d'un ajout
        /// </summary>
        protected virtual void ExecuteAdd() { }
        /// <summary>
        /// Excution d'un effacement
        /// </summary>
        protected virtual void ExecuteDel() { }
        /// <summary>
        /// Refreshes the DataGridView
        /// </summary>
        protected virtual void RefreshDGV() { }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise la visibilit du menu de suppression
        /// </summary>
        public bool DeleteVisible { get { return tsMenu_Del.Visible; } set { tsMenu_Del.Visible = value; } }
        /// <summary>
        /// Gets or sets the search options
        /// </summary>
        protected SearchOption SearchOption { get { return vSearchOption; } set { vSearchOption = value; } }
        #endregion
    }
    /// <summary>
    /// Indicates the type of search
    /// </summary>
    public enum SearchOption
    {
        /// <summary>
        /// Seek term which begins with another term
        /// </summary>
        BeginsWith,
        /// <summary>
        /// Seek term which contains another term
        /// </summary>
        Contains,
    }
}